
#Load libraries and source file

require(Hmisc)
library(readr)

source("src/functions.R")


###IMPORT DATA AND DEFINE VARIABLES#################################################

###############OBTAIN ET0 based on Hargreaves calibrated equation###################

#1. load data. Example for weather station 7002A----------------------------------

TASdatahist = data.frame(read_delim("data/TASdatahist-7002A.csv", ";", escape_double = FALSE, col_types = cols(Date = col_date(format = "%Y-%m-%d"))))
TASdatarcp45 = data.frame(read_delim("data/TASdatarcp45-7002A.csv", ";", escape_double = FALSE, col_types = cols(Date = col_date(format = "%Y-%m-%d"))))
TASdatarcp85 = data.frame(read_delim("data/TASdatarcp85-7002A.csv", ";", escape_double = FALSE, col_types = cols(Date = col_date(format = "%Y-%m-%d"))))

#data frames contain information about daily maximum and minimun tempeartures: observed temperatures (tmax_ref, tmin_ref) and individual regionalized models (tmax_NAME and tmin_NAME)

#2. Parameters for obtain Hargreaves calibrated ET0------------------------------- 
#Calibrated parameters se obtuvieron en Gomariz-Castillo et al. (2018)

latitude = 37.403
a = 0.08233651
b = 1.1202406

#3. Define variable names----------------------------------------------------------
# Individual regionalized models for HISTORICAL and RCP scenarios

modhist<-c("ref","ACCESS1_0","ACCESS1_3","bcc_csm1_1","bcc_csm1_1_m","BNU_ESM","CMCC_CESM","CMCC_CM","CMCC_CMS","CNRM_CM5","inmcm4","IPSL_CM5A_MR","MIROC_ESM","MIROC5","MPI_ESM_LR","MPI_ESM_MR","MRI_CGCM3")

modrcp<-c("ACCESS1_0","bcc_csm1_1","BNU_ESM","CNRM_CM5" ,"inmcm4","MIROC_ESM","MIROC5","MPI_ESM_LR","MPI_ESM_MR","MRI_CGCM3")

#4. Obtain ET0 for all scenarios and reference data--------------------------------

#HISTORICAL scenario:
for(name in modhist){
  tmax = TASdatahist[,paste0("tmax_",name)]
  tmin = TASdatahist[,paste0("tmin_",name)]
  day = TASdatahist[,"Date"]
  TASdatahist[,paste0("ET0_",name)] = fun.ETPHargreaves(latitude=latitude,tmax=tmax,tmin=tmin,day=day,Corr=T,a=a,b=b)$ET0h
}

#RCP45 scenario:
for(name in modrcp){
  tmax = TASdatarcp45[,paste0("tmax_",name)]
  tmin = TASdatarcp45[,paste0("tmin_",name)]
  day = TASdatarcp45[,"Date"]
  TASdatarcp45[,paste0("ET0_",name)] = fun.ETPHargreaves(latitude=latitude,tmax=tmax,tmin=tmin,day=day,Corr=T,a=a,b=b)$ET0h
}

#RCP85 scenario:
for(name in modrcp){
  tmax = TASdatarcp85[,paste0("tmax_",name)]
  tmin = TASdatarcp85[,paste0("tmin_",name)]
  day = TASdatarcp85[,"Date"]
  TASdatarcp85[,paste0("ET0_",name)] = fun.ETPHargreaves(latitude=latitude,tmax=tmax,tmin=tmin,day=day,Corr=T,a=a,b=b)$ET0h
}


###############EVALUATE SERIES AND OBTAIN ENSEMBLE MODELS FROM ET0 DATA###################

#1. Select data for ensemble and rename columns------------------------------------

modhist<- modhist[-1]# Delete ref value

datahist = TASdatahist[,c("idest","Date","ET0_ref",paste0("ET0_",modhist))]
names(datahist) = c("idest","Date","ref",modhist)

datarcp45 = TASdatarcp45[,c("idest","Date",paste0("ET0_",modrcp))]
names(datarcp45) = c("idest","Date",modrcp)

datarcp85 = TASdatarcp85[,c("idest","Date",paste0("ET0_",modrcp))]
names(datarcp85) = c("idest","Date",modrcp)


#2. divide data into training and test data----------------------------------------
# Case of 2/3 for training and 1/3 test data

timeseries=seq(min(datahist$Date),max(datahist$Date),by="day")

CalIni=min(timeseries)
CalEnd=timeseries[(length(timeseries) * (2 / 3))] + 1
TestIni=CalEnd + 1
TestEnd=max(timeseries)

# For machine learning ensembles (RF and SVM) function calibra sus parámetros. Para ello se realiza una CV basada en K-Folds. En ella, se generan índices basados en agrupaciones definidas en una columna del dataframe histórico. In this case, hemos utilizado una simple option for calibration-validation based on split by decaces. Puede modificarse por otro tipo de particiones, pero el incremento del tiempo de cómputo puede aumentar considerablemente

datahist$year2 = paste0(substr(datahist[,"Date"],3,3),"0")

#3. ALTERNATIVE FOR CHECK PROCESS: Select data for years 1990 to 1993

timeseries3=seq(as.Date("1990-01-01"),as.Date("1992-12-31"),by="day")
datahist3 = datahist[datahist$Date %in% timeseries3,]

#4. divide data into training and test data----------------------------------------
# Case of 2/3 for training and 1/3 test data

CalIni3=min(timeseries3)
CalEnd3=timeseries3[(length(timeseries3) * (2 / 3))]
TestIni3=CalEnd3 + 1
TestEnd3=max(timeseries3)

#Column for K-Folds
datahist3$year2 = substr(datahist3[,"Date"],3,4)

#5. Generate ensembles-------------------------------------------------------------

#Function for ensemble
#ensemble = fun.ensembles(datahist=datahist,datarcp1=datarcp45,datarcp2=datarcp85,ref="ref",date="Date",CalIni=CalIni,CalEnd=CalEnd,TestIni=TestIni,TestEnd=TestEnd,modhist=modhist,modensembles=modrcp,groupK="year2",parallel=T,numcores=6,est.caltest = F)

#Alternative for five years (for test code)
ensemble3 = fun.ensembles(datahist=datahist3,datarcp1=datarcp45,datarcp2=datarcp85,ref="ref",date="Date",CalIni=CalIni3,CalEnd=CalEnd3,TestIni=TestIni3,TestEnd=TestEnd3,modhist=modhist,modensembles=modrcp,groupK="year2",parallel=T,numcores=3,est.caltest = T)

names(ensemble3)

ensemble3$validation_test
